{ ****************************************************************** }
{                                                                    }
{   VCL component TFormResizer:  Resizes all components on a form    }
{                                                                    }
{   For automatic resizing of a form's controls when the form        }
{   resizes, do the following:                                       }
{                                                                    }
{   1.  Add a TFormResizer component to the form.                    }
{                                                                    }
{   2.  In the form's OnCreate handler, add a call                   }
{       to the component's InitializeForm method, e.g.,              }
{          FormResizer1.InitializeForm;                              }
{                                                                    }
{   3.  In the form's OnResize handler, add a call                   }
{       to the component's ResizeAll method, e.g.,                   }
{          FormResizer1.ResizeAll;                                   }
{                                                                    }
{   4.  If you want to add custom resizing behavior for a particular }
{       component, add a handler for the TFormResizer's              }
{       OnPostResize event, which is fired after a component         }
{       is resized.  Its arguments are                               }
{          Sender: TObject -- as usual                               }
{          Comp : TControl -- the component that has                 }
{                              just been resized                     }
{          HeightRatio : Double -- ratio of the form's initial       }
{                                  height to its current height.     }
{                                  E.g., if the form's height has    }
{                                  doubled since the call to         }
{                                  InitializeForm, then HeightRatio  }
{                                  is 2.0.                           }
{          WidthRatio : Double -- as above for form's width          }
{       Two quick samples:                                           }
{          if Comp = Button1 then                                    }
{             if WidthRatio < 0.8 then                               }
{                Button1.Caption := ...                              }
{          if Comp is TMyClass then                                  }
{             TMyClass(Comp).SomeProperty := ...                     }
{                                                                    }
{   5.  If you also want the TFormResizer to enforce the             }
{       form's initial size as its minimum size, set the             }
{       EnforceMinSize property to True.  (The user will             }
{       still be able to minimize the form into an icon              }
{       using the Windows minimize button.)                          }
{                                                                    }
{   Version 1.1 (Jan. 1999)                                          }
{                                                                    }
{   Copyright  1999 by David A. Price, dprice@erols.com             }
{   http://www.compcreate.com                                        }
{   This file may be redistributed freely if not altered             }
{   Suggestions are welcome                                          }
{   Offered "AS IS" -- no warranties                                 }
{                                                                    }
{ ****************************************************************** }

unit Easysize;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms, Graphics, StdCtrls;

type
  EResizerException = class(Exception);

  TPostResizeEvent = procedure (Sender: TObject;
                                Comp : TControl;
                                HeightRatio, WidthRatio : Double) of object;

  TFormResizer = class(TComponent)
    private
      { Private fields of TFormResizer }
      FEnforceMinSize : Boolean;
      FOnPostResize : TPostResizeEvent;
      InitHeightOwner, InitWidthOwner : Integer;
      ResizerList : TList;

      { Private methods of TFormResizer }

    protected
      { Protected fields of TFormResizer }
        AlreadyInitialized : Boolean;

      { Protected methods of TFormResizer }
        procedure Loaded; override;
        procedure Notification(AComponent : TComponent; Operation : TOperation); override;
        procedure PostResize(Sender: TObject; Comp : TControl;
                             HeightRatio, WidthRatio : Double); virtual;

    public
      { Public fields and properties of TFormResizer }

      { Public methods of TFormResizer }
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
        procedure InitializeForm; virtual;
        procedure ResizeAll; virtual;
    published
      { Published properties of TFormResizer }
        property EnforceMinSize : Boolean
           read FEnforceMinSize write FEnforceMinSize
           default False;
        property OnPostResize : TPostResizeEvent
           read FOnPostResize write FOnPostResize;
  end;

procedure Register;

implementation

procedure Register;
begin
     { Register TFormResizer with Additional as its
       default page on the Delphi component palette }
     RegisterComponents('Additional', [TFormResizer]);
end;

{ ****************************************************************** }
{ TSingleResizer object                                              }
{ ****************************************************************** }

type
  TSingleResizer = class(TObject)
    private
      { Private fields of TSingleResizer }
        FComponent : TControl;
        FFlexHeight : Boolean;
        FFlexHorizPos : Boolean;
        FFlexVertPos : Boolean;
        FFlexWidth : Boolean;

      { Private methods of TSingleResizer }

    protected
      { Protected fields of TSingleResizer }
        { Initial dimensions of the linked component and its owner }
        InitHeight : Integer;
        InitHeightOwner : Integer;
        InitLeft : Integer;
        InitTop : Integer;
        InitWidth : Integer;
        InitWidthOwner : Integer;
        { Owner of the linked component }
        Owner : TControl;

      { Protected methods of TSingleResizer }

    public
      { Public fields and properties of TSingleResizer }
        { Component to be resized when form resizes }
        property Component : TControl read FComponent;
        { Scale component's height? }
        property FlexHeight : Boolean
             read FFlexHeight write FFlexHeight;
        { Scale component's horizontal position? }
        property FlexHorizPos : Boolean
             read FFlexHorizPos write FFlexHorizPos;
        { Scale component's vertical position? }
        property FlexVertPos : Boolean
             read FFlexVertPos write FFlexVertPos;
        { Scale component's width? }
        property FlexWidth : Boolean
             read FFlexWidth write FFlexWidth;

      { Public methods of TSingleResizer }
        constructor Create(LinkedControl, ParentForm : TControl);
        destructor Destroy; override;
        { Resize the linked component }
        procedure Resize(OwnerHeight, OwnerWidth,
                         HeightRatio, WidthRatio : Double);
        { Remove the link to the component }
        procedure Unlink;
  end;

constructor TSingleResizer.Create(LinkedControl, ParentForm : TControl);
begin
     inherited Create;
     FComponent := LinkedControl;
     if FComponent = nil then
	begin
	raise EResizerException.Create('TSingleResizer created without a component link');
        Exit
        end;
     Owner := ParentForm;
     if Owner = nil then
        begin
        raise EResizerException.Create('TSingleResizer created without a parent form link');
        FComponent := nil;
        Exit
        end;
     { Obtain the linked component's dimensions }
     InitHeight := FComponent.Height;
     InitLeft := FComponent.Left;
     InitTop := FComponent.Top;
     InitWidth := FComponent.Width;
     { Obtain the linked component's owner's dimensions }
     InitHeightOwner := (Owner as TControl).Height;
     InitWidthOwner := (Owner as TControl).Width;
     { By default, we adjust both size and position }
     FFlexHeight := True;
     FFlexHorizPos := True;
     FFlexVertPos := True;
     FFlexWidth := True;
     { But do not adjust width of labels (to avoid
       interaction of label auto-sizing with re-sizing) }
     if FComponent is TCustomLabel then
        FFlexWidth := False;
end;

destructor TSingleResizer.Destroy;
begin
     inherited Destroy;
end;

{ Resize the linked component }
procedure TSingleResizer.Resize(OwnerHeight, OwnerWidth,
                                HeightRatio, WidthRatio : Double);
var
   NewHeight, NewLeft, NewTop, NewWidth : Integer;
begin
     if FComponent = nil then
        Exit; { not linked to a component }
     if FComponent.Align = alClient then
        Exit; { component resizes itself to client area of parent }
     { Get current dimensions }
     NewHeight := FComponent.Height;
     NewLeft := FComponent.Left;
     NewTop := FComponent.Top;
     NewWidth := FComponent.Width;
     { Is form back to original size?  If so, restore
       component to exact original size to avoid
       rounding error. }
     if (Round(OwnerHeight) = InitHeightOwner)
        and (Round(OwnerWidth) = InitWidthOwner) then
        begin
        if FlexHeight then
           NewHeight := InitHeight;
        if FlexHorizPos then
           NewLeft := InitLeft;
        if FlexVertPos then
           NewTop := InitTop;
        if FlexWidth then
           NewWidth := InitWidth;
        FComponent.SetBounds(NewLeft, NewTop, NewWidth, NewHeight);
        Exit;
        end;
     { Component re-sizing and re-scaling }
     if FlexHeight then
        begin
        { Adjust component's Height property }
        NewHeight := Round(InitHeight * HeightRatio);
        end;
     if FlexWidth then
        begin
        { Adjust component's Width property }
        NewWidth := Round(InitWidth * WidthRatio);
        end;
     if FlexHorizPos then
        begin
        { Adjust component's Left property }
        NewLeft := Round(InitLeft * WidthRatio);
        end;
     if FlexVertPos then
        begin
        { Adjust component's Top property }
        NewTop := Round(InitTop * HeightRatio);
        end;
     FComponent.SetBounds(NewLeft, NewTop, NewWidth, NewHeight);
end;

procedure TSingleResizer.Unlink;
begin
     FComponent := nil
end;

{ ****************************************************************** }
{ TFormResizer component                                             }
{ ****************************************************************** }

constructor TFormResizer.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FEnforceMinSize := False;
     AlreadyInitialized := False;
     ResizerList := TList.Create;
     if Owner = nil then
        begin
        raise EResizerException.Create('TFormResizer created without an owner');
        Exit
        end;
     if not (Owner is TControl) then
        begin
        raise EResizerException.Create('TFormResizer owner must be a panel or form');
        Exit
        end;
end;

destructor TFormResizer.Destroy;
var
   I : Integer;
begin
     for I := 0 to ResizerList.Count - 1 do
         TSingleResizer(ResizerList.Items[I]).Free;
     ResizerList.Free;
     inherited Destroy;
end;

procedure TFormResizer.InitializeForm;
var
   I : Integer;
   R : TSingleResizer;
begin
     if AlreadyInitialized then
        Exit;
     AlreadyInitialized := True;
     { Get initial height and width of owner form in case
       we are to enforce it as a minimum size
       (i.e., EnforceMinSize = True) }
     InitHeightOwner := TControl(Owner).Height;
     InitWidthOwner := TControl(Owner).Width;
     { Create a TSingleResizer for each control on the form }
     for I := 0 to Owner.ComponentCount - 1 do
         if Owner.Components[I] is TControl then
            begin
            R := TSingleResizer.Create(Owner.Components[I] as TControl,
                                       Owner as TControl);
            ResizerList.Add(R)
            end;
end;

procedure TFormResizer.Loaded;
begin
     inherited Loaded;
end;

{ Called when a component on the form has been deleted }
procedure TFormResizer.Notification(AComponent : TComponent; Operation : TOperation);
var
   I : Integer;
begin
     inherited Notification(AComponent, Operation);
     if Operation <> opRemove then
        Exit;

     { Has a component referenced by one of the TSingleResizer objects
       been deleted?  If so, remove the reference. }
     for I := 0 to ResizerList.Count - 1 do
         if TSingleResizer(ResizerList.Items[I]).Component = AComponent then
            TSingleResizer(ResizerList.Items[I]).Unlink;
end;

{ Fire an OnPostResize event after a component has been resized. }
procedure TFormResizer.PostResize(Sender: TObject; Comp : TControl;
                                  HeightRatio, WidthRatio : Double);
begin
     if Assigned(FOnPostResize) then
        FOnPostResize(Sender, Comp, HeightRatio, WidthRatio)
end;

procedure TFormResizer.ResizeAll;
var
   I : Integer;
   OwnerHeight, OwnerWidth, HeightRatio, WidthRatio : Double;
begin
     { Enforce initial height and width of owner form as minimum size? }
     if EnforceMinSize then
        begin
        if TControl(Owner).Height < InitHeightOwner then
           TControl(Owner).Height := InitHeightOwner;
        if TControl(Owner).Width < InitWidthOwner then
           TControl(Owner).Width := InitWidthOwner
        end;
     { Determine ratio of current owner size to initial form size }
     OwnerHeight := 1.0 * TControl(Owner).Height;
     OwnerWidth := 1.0 * TControl(Owner).Width;
     HeightRatio := OwnerHeight/(1.0 * InitHeightOwner);
     WidthRatio := OwnerWidth/(1.0 * InitWidthOwner);
     { Fire all TSingleResizer components on the form }
     for I := 0 to ResizerList.Count - 1 do
         begin
         TSingleResizer(ResizerList.Items[I]).Resize(OwnerHeight, OwnerWidth,
                        HeightRatio, WidthRatio);
         { Apply any post-resize behavior defined by descendant class
           in PostResize method or by application's OnPostResize handler. }
         PostResize(Self, TSingleResizer(ResizerList.Items[I]).Component,
                    HeightRatio, WidthRatio);
         end
end;


end.
